/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "FlashItem.h"

extern int screenWidth;
extern int screenHeight;

extern int bitsperpixel;

#define GET_X_LPARAM(lp) ((int)(short)LOWORD(lp))
#define GET_Y_LPARAM(lp) ((int)(short)HIWORD(lp))

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

FlashItem::FlashItem(const string &name):actualheight(0),actualwidth(0),actualx(0),actualy(0)
{
	this->name = name;

	hWnd = 0;
	hWndFlash = 0;
	hInstance = 0;

	memoryDC = 0;
	memoryBitmap = 0;

	alwaysOnTop = false;
	visible = false;
	startHidden = false;
}

FlashItem::~FlashItem()
{
	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);

	if (hWndFlash)
	{
		DestroyWindow(hWndFlash);
		hWndFlash = 0;
	}

	if(IsWindow(hWnd))
	{
		DestroyWindow(hWnd);
		hWnd = 0;
	}
}

void FlashItem::load(HINSTANCE hInstance, const string &flashpath)
{
	//Initial Repositioning
	actualx = GetxRCCoord(name, "X", 0, screenWidth);
	actualy = GetxRCCoord(name, "Y", 0, screenHeight);
	actualwidth = GetxRCDimen(name, "Width", 64, screenWidth);
	actualheight = GetxRCDimen(name, "Height", 64, screenHeight);

	startHidden = GetxRCBool(name, "StartHidden");

	hWnd = CreateWindowEx(  0 ,
							"xFlash",
							name.c_str(),
							WS_POPUP,
							actualx, actualy,		
							actualwidth,actualheight,
							GetLitestepWnd(), 
							0,
							hInstance,
							this);

	hWndFlash = CreateWindowEx( 0,
							"AtlAxWin",
							flashpath.c_str(),
							WS_CHILD | WS_VISIBLE,
							0, 0,		
							actualwidth,actualheight,
							hWnd, 
							0,
							hInstance,
							this);

	setAlwaysOnTop(GetxRCBool(name, "AlwaysOnTop"), true);

	string maskImage = GetxRCString(name, "MaskImage", "");
	if ( !maskImage.empty() )
	{
		memoryBitmap = LoadLSImage(maskImage.c_str(), 0);

		GetLSBitmapSize(memoryBitmap, &maskWidth, &maskHeight);

		memoryDC = CreateCompatibleDC(NULL);
		SelectObject(memoryDC, memoryBitmap);
	}
 
	if ( hWnd )
	{
		SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

		updateRegion();
	}
	else
	{
		delete this;
	}
}

void FlashItem::setAlwaysOnTop(bool setalwaysOnTop, bool firstSetup)
{
	bool realChange = ( alwaysOnTop != setalwaysOnTop );

	alwaysOnTop = setalwaysOnTop;
	
	if ( realChange || firstSetup )
	{
		ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
		SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
		ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
	}

	SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
						0, 0, 0, 0,
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
}

void FlashItem::hide()
{
	ShowWindow(hWnd, SW_HIDE);

	visible = false;
}

void FlashItem::show()
{
	ShowWindow(hWnd, SW_SHOW);

	visible = true;
}

void FlashItem::reposition(int type, int newx, int newy, int newwidth, int newheight, bool repositionBy)
{
	RECT rectPosition;
	rectPosition.left = actualx;
	rectPosition.right = actualx+actualwidth;
	rectPosition.top = actualy;
	rectPosition.bottom = actualy+actualheight;

	POINT posHolder;
	posHolder.x = rectPosition.left;
	posHolder.y = rectPosition.top;

	POINT sizeHolder;
	sizeHolder.x = (rectPosition.right-rectPosition.left);
	sizeHolder.y = (rectPosition.bottom-rectPosition.top);

	if (repositionBy)
	{
		if (type != 2)
		{
			newx = rectPosition.left + newx;
			newy = rectPosition.top + newy;
		}
		if (type != 1)
		{
			newwidth = (rectPosition.right-rectPosition.left) + newwidth;
			newheight = (rectPosition.bottom-rectPosition.top) + newheight;
		}
	}

	if ( type == 1 )
	{
		SetWindowPos(hWnd, 0, newx, newy, 0, 0,
						SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
	}
	else if ( type == 2 )
	{
		SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);
	}
	else if ( type == 3 )
	{
		SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
						SWP_NOACTIVATE | SWP_NOZORDER);
	}
}

void FlashItem::updateRegion()
{	
	if ( memoryBitmap != 0 && memoryDC != 0 )
	{
		RECT rectPosition;
		GetClientRect(hWnd, &rectPosition);

		HDC tmpDC = CreateCompatibleDC(NULL);
		HBITMAP tmpBmp = CreateBitmap(rectPosition.right, rectPosition.bottom, 1, bitsperpixel, NULL);
		HBITMAP oldBMP = (HBITMAP) SelectObject(tmpDC, tmpBmp);

		StretchBlt(tmpDC, 0, 0, rectPosition.right, rectPosition.bottom, memoryDC, 0, 0, maskWidth, maskHeight, SRCCOPY);

		tmpBmp = (HBITMAP) SelectObject(tmpDC, oldBMP);

		HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
		HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(windowRgn, region, NULL, RGN_COPY);

		SetWindowRgn(hWnd, windowRgn, true);	

		DeleteObject(windowRgn);
		DeleteObject(region);
		DeleteObject(tmpBmp);
		DeleteDC(tmpDC);
	}
}

LRESULT FlashItem::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	FlashItem *flashitem = NULL;

	if (message == WM_NCCREATE)
	{
		flashitem = (FlashItem *) ((CREATESTRUCT *) lParam)->lpCreateParams;
		flashitem->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) flashitem);
	}
	else
	{
		flashitem = (FlashItem *) GetWindowLong(hWnd, 0);
	}

	if (flashitem)
	{
		LRESULT lResult = 0;

		if ( flashitem->onWindowMessage(message, wParam, lParam, lResult) )
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}


bool FlashItem::onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult)
{
	switch(message)
	{
		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_SIZE:
		{
			SetWindowPos(hWndFlash, 0, 0, 0, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam),
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);

			updateRegion();
			return true;
		}
	}

	return false;
}
