/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "Utility.h"

// modify window style
void ModifyStyle(HWND hWnd, DWORD removeStyle, DWORD addStyle)
{
	SetWindowLong(hWnd, GWL_STYLE, (GetWindowLong(hWnd, GWL_STYLE) & ~removeStyle) | addStyle);
}

// trim leading and trailing whitespace
string trimit(const string &toTrim)
{
	int start = 0;
	int end = toTrim.length() - 1;
	
	while(start <= end && isspace(toTrim[start]))
		start++;
	
	while(start <= end && isspace(toTrim[end]))
		end--;
	
	return toTrim.substr(start, end - start + 1);
}

// retrieve handle of the desktop window
HWND GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);

	if(hWnd)
		return hWnd;

	return GetDesktopWindow();
}

// retrieve a list of names from step.rc
StringList GetxRCNameList(const string &uniqueKey)
{
	StringList resultList;
	FILE * f = LCOpen(NULL);

	if(f != NULL)
	{
		char lineBuffer[MAX_LINE_LENGTH];

		while(LCReadNextConfig(f, uniqueKey.c_str(), lineBuffer, MAX_LINE_LENGTH))
		{
			char prefix[64], rest[MAX_LINE_LENGTH];
			char *buffers[] = {prefix};
			
			LCTokenize(lineBuffer, buffers, 1, rest);
			
			resultList.insert(resultList.end(), rest);
		}

		LCClose(f);
	}

	return resultList;
}

// parse a coordinate specification
int xParseCoord(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trimit(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool realnegative = false;
	bool center = false;
	UINT i = 0;
	int value = 0;

	if ( strstr(strVal.c_str(), "@") != NULL )
		i = i+2;

	if (strVal[i] == '-')
	{
		negative = true;
		i++;
	}
	else if (strVal[i] == '~')
	{
		realnegative = true;
		i++;
	}
	else if (strVal[i] == '+')
	{
		i++;
	}

	while (i < strVal.length())
	{
		if(strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if(strVal[i] == 'c' || strVal[i] == 'C')
				center = true;

			break;
		}

		i++;
	}

	if (center)
	{
		if (realnegative)
			negative = true;

		if (negative)
			value = (maxVal/2) - value;
		else
			value = (maxVal/2) + value;
	}
	else if (realnegative)
	{
		value = -value;
	}
	else if (negative)
	{
		value = maxVal - value;
	}

	return value;
}

// retrieve a coordinate from step.rc, accounting for negative values and centering
int GetxRCCoord(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(uniqueKey, baseName, "");
	return xParseCoord(strVal, defaultVal, maxVal);
}

// parse a dimension
int xParseDimen(const string &aString, int defaultVal, int maxVal)
{
	string strVal = trimit(aString);

	if (strVal.length() == 0)
		return defaultVal;

	bool negative = false;
	bool percentage = false;
	UINT i = 0;
	int value = 0;

	if ( strstr(strVal.c_str(), "@") != NULL )
		i= i+2;

	if (strVal[i] == '-')
	{
		negative = true;
		i++;
	}

	while (i < strVal.length())
	{
		if (strVal[i] >= '0' && strVal[i] <= '9')
		{
			value = (value * 10) + (strVal[i] - '0');
		}
		else
		{
			if (strVal[i] == '%')
				percentage = true;

			break;
		}

		i++;
	}

	if (percentage)
		value = (value * maxVal) / 100;

	if (negative)
		value = maxVal - value;

	return value;
}

// retrieve a dimension (absolute value or percentage) from step.rc
int GetxRCDimen(const string &uniqueKey, const string &baseName, int defaultVal, int maxVal)
{
	string strVal = GetxRCString(uniqueKey, baseName, "");
	return xParseDimen(strVal, defaultVal, maxVal);
}

// retrieve a boolean value from the step.rc
bool GetxRCBool(const string &uniqueKey, const string &baseName, bool defaultVal)
{
	string key = uniqueKey + baseName;
	return GetRCBoolDef(key.c_str(), defaultVal) ? true : false;
}

// retrieve a string value from step.rc
string GetxRCString(const string &uniqueKey, const string &baseName, const string &defaultVal)
{
	string key = uniqueKey + baseName;
	char buffer[MAX_LINE_LENGTH] = {0};
	GetRCString(key.c_str(), buffer, defaultVal.c_str(), MAX_LINE_LENGTH);
	return string(buffer);
}