/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "flashitem.h"
#include "VerInfo.h"

#define StringCchCopy(a, b, c) strncpy((a), (c), (b))
#define StringCchLength(a, b, c) *(c) = strlen((a)), S_OK
#define StringCchPrintf _snprintf

void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs);
void BangMove(HWND hwndCaller, LPCSTR pszArgs);
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs);
void BangReposition(HWND hwndCaller, LPCSTR pszArgs);
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs);
void BangResize(HWND hwndCaller, LPCSTR pszArgs);
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs);
void BangHide(HWND hwndCaller, LPCSTR pszArgs);
void BangShow(HWND hwndCaller, LPCSTR pszArgs);
void BangToggle(HWND hwndCaller, LPCSTR pszArgs);

int screenWidth = 0;
int screenHeight = 0;

int bitsperpixel = 8;

FlashItem *lookupFlashItem( const string &name );

HINSTANCE hInstance;
HWND messageHandler;

FlashItemList flashitemList;

int lsMessages[] = {
	LM_GETREVID,
	0
};

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS | CS_DBLCLKS ;
	wc.lpfnWndProc = FlashItem::windowProcedure;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(FlashItem *);
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xFlash";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xFlashMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xFlashMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;

	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	screenWidth = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);

	CString Response = "";
	DEVMODE DevMode;
	DevMode.dmSize=sizeof(DEVMODE);
		
	if (EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&DevMode))
	{
		Response.Format("%ld", DevMode.dmBitsPerPel);
	}
	else
	{
		Response = "8";
	}

	bitsperpixel = atoi(Response);

	AddBangCommand("!FlashAlwaysOnTop", BangAlwaysOnTop);
	AddBangCommand("!FlashHide", BangHide);
	AddBangCommand("!FlashMove", BangMove);
	AddBangCommand("!FlashMoveBy", BangMoveBy);
	AddBangCommand("!FlashReposition", BangReposition);
	AddBangCommand("!FlashRepositionBy", BangRepositionBy);
	AddBangCommand("!FlashResize", BangResize);
	AddBangCommand("!FlashResizeBy", BangResizeBy);
	AddBangCommand("!FlashShow", BangShow);
	AddBangCommand("!FlashToggle", BangToggle);

	void* init = GetProcAddress(LoadLibrary("atl"), "AtlAxWinInit"); 
	_asm call init;

	StringList flashItems = GetxRCNameList("*xFlash");

	char name[64], path[MAX_LINE_LENGTH];
	char *buffers[] = {name, path};
	
	for (StringListIterator sit = flashItems.begin(); sit != flashItems.end(); sit++)
	{
		LCTokenize((*sit).c_str(), buffers, 2, NULL);

		FlashItem *flashitem = new FlashItem((string)name);
		flashitem->load(hInstance, (string)path);
	
		if ( flashitem )
		{	
			flashitemList.insert(flashitemList.end(), flashitem);

			//Initial Show
			if (!flashitem->isStartHidden())
				flashitem->show();
			else
				flashitem->hide();
		}
	}

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	RemoveBangCommand("!FlashAlwaysOnTop");
	RemoveBangCommand("!FlashHide");
	RemoveBangCommand("!FlashMove");
	RemoveBangCommand("!FlashMoveBy");
	RemoveBangCommand("!FlashReposition");
	RemoveBangCommand("!FlashRepositionBy");
	RemoveBangCommand("!FlashResize");
	RemoveBangCommand("!FlashResizeBy");
	RemoveBangCommand("!FlashShow");
	RemoveBangCommand("!FlashToggle");

	for (FlashItemListIterator lit = flashitemList.begin(); lit != flashitemList.end(); lit++)
	{
		(*lit)->hide();
		delete *lit;
	}

	flashitemList.clear();

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xFlash", hInstance);
	UnregisterClass("xFlashMessageHandler", hInstance);
}

FlashItem *lookupFlashItem( const string &name )
{
	for (FlashItemListIterator lit = flashitemList.begin(); lit != flashitemList.end(); lit++)
	{
		if (_stricmp(name.c_str(), (*lit)->getName().c_str()) == NULL)
			return *lit;
	}

	return NULL;
}

// !FlashAlwaysOnTop <flashitem> <on|true, off|false, toggle>
void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], option[16];
	char *buffers[] = {name, option};

	LCTokenize(pszArgs, buffers, 2, NULL);

	FlashItem *flashitem = lookupFlashItem(name);

	if (flashitem)
	{
		if (_stricmp(option, "false") == NULL || _stricmp(option, "off") == NULL)
			flashitem->setAlwaysOnTop(false);
		else if (_stricmp(option, "toggle") == NULL || _stricmp(option, "switch") == NULL)
			flashitem->setAlwaysOnTop(!flashitem->isAlwaysOnTop());
		else
			flashitem->setAlwaysOnTop(true);
	}
}

// !FlashShow <flashitem>
void BangShow(HWND hwndCaller, LPCSTR pszArgs)
{
	FlashItem *flashitem = lookupFlashItem(pszArgs);
	
	if (flashitem)
	{
		flashitem->show();
	}
}

// !FlashHide <flashitem>
void BangHide(HWND hwndCaller, LPCSTR pszArgs)
{
	FlashItem *flashitem = lookupFlashItem(pszArgs);
	
	if (flashitem)
	{
		flashitem->hide();
	}
}

// !FlashToggle <flashitem>
void BangToggle(HWND hwndCaller, LPCSTR pszArgs)
{
	FlashItem *flashitem = lookupFlashItem(pszArgs);
	
	if (flashitem)
	{
		if ( flashitem->isVisible() )
			flashitem->hide();
		else
			flashitem->show();
	}
}

// !FlashMove <flashitem> <x> <y>
void BangMove(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16];
	char *buffers[] = {name, A, B};

	if (LCTokenize(pszArgs, buffers, 3, NULL) >= 3)
	{
		FlashItem *flashitem = lookupFlashItem(name);

		if (flashitem)
		{
			int x = xParseCoord(A, 0, screenWidth);
			int y = xParseCoord(B, 0, screenHeight);
				
			flashitem->reposition(1, x, y, 0, 0);
		}
	}
}

// !FlashMoveBy <flashitem> <x> <y>
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16];
	char *buffers[] = {name, A, B};

	if (LCTokenize(pszArgs, buffers, 3, NULL) >= 3)
	{
		FlashItem *flashitem = lookupFlashItem(name);

		if (flashitem)
		{
			flashitem->reposition(1, atoi( A ), atoi( B ), 0, 0, true);
		}
	}
}

// !FlashReposition <flashitem> <x> <y> <width> <height>
void BangReposition(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	if(LCTokenize(pszArgs, buffers, 5, NULL) >= 5)
	{
		FlashItem *flashitem = lookupFlashItem(name);

		if (flashitem)
		{
			int x = xParseCoord(A, 0, screenWidth);
			int y = xParseCoord(B, 0, screenHeight);
			int w = xParseDimen(C, 0, screenWidth);
			int h = xParseDimen(D, 0, screenHeight);
				
			flashitem->reposition(3, x, y, w, h);
		}
	}
}

// !FlashRepositionBy <flashitem> <cx> <cy> <cwidth> <cheight>
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16], C[16], D[16];
	char *buffers[] = {name, A, B, C, D};

	if (LCTokenize(pszArgs, buffers, 5, NULL) >= 5)
	{
		FlashItem *flashitem = lookupFlashItem(name);
		
		if (flashitem)
		{
			int x = atoi( A );
			int y = atoi( B );
			int w = atoi( C );
			int h = atoi( D );

			flashitem->reposition(3, x, y, w, h, true);
		}
	}
}


// !FlashResize <flashitem> <width> <height>
void BangResize(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16];
	char *buffers[] = {name, A, B};

	if (LCTokenize(pszArgs, buffers, 3, NULL) >= 3)
	{
		FlashItem *flashitem = lookupFlashItem(name);

		if (flashitem)
		{
			int w = xParseDimen(A, 0, screenWidth);
			int h = xParseDimen(B, 0, screenHeight);
				
			flashitem->reposition(2, 0, 0, w, h);
		}
	}
}

// !FlashResizeBy <flashitem> <cx> <cy>
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs)
{
	char name[64], A[16], B[16];
	char *buffers[] = {name, A, B};

	if (LCTokenize(pszArgs, buffers, 3, NULL) >= 3)
	{
		FlashItem *flashitem = lookupFlashItem(name);

		if (flashitem)
		{
			int cx = atoi( A );
			int cy = atoi( B );

			flashitem->reposition(2, 0, 0, cx, cy, true);
		}
	}
}